@@ -370,10 +370,10 @@ static int net_getdomainsid(struct net_context *c, int argc, const char **argv)
 }
 
 static bool search_maxrid(struct pdb_search *search, const char *type,
-			  uint32 *max_rid)
+			  uint32_t *max_rid)
 {
 	struct samr_displayentry *entries;
-	uint32 i, num_entries;
+	uint32_t i, num_entries;
 
 	if (search == NULL) {
 		d_fprintf(stderr, _("get_maxrid: Could not search %s\n"), type);
@@ -387,9 +387,9 @@ static bool search_maxrid(struct pdb_search *search, const char *type,
 	return true;
 }
 
-static uint32 get_maxrid(void)
+static uint32_t get_maxrid(void)
 {
-	uint32 max_rid = 0;
+	uint32_t max_rid = 0;
 
 	if (!search_maxrid(pdb_search_users(talloc_tos(), 0), "users", &max_rid))
 		return 0;
@@ -407,7 +407,7 @@ static uint32 get_maxrid(void)
 
 static int net_maxrid(struct net_context *c, int argc, const char **argv)
 {
-	uint32 rid;
+	uint32_t rid;
 
 	if (argc != 0) {
 	        d_fprintf(stderr, "%s net maxrid\n", _("Usage:"));
@@ -743,6 +743,14 @@ static struct functable net_func[] = {
 		   "'net serverid' commands.")
 	},
 
+	{	"notify",
+		net_notify,
+		NET_TRANSPORT_LOCAL,
+		N_("notifyd client code"),
+		N_("  Use 'net help notify' to get more information about "
+		   "'net notify' commands.")
+	},
+
 #ifdef WITH_FAKE_KASERVER
 	{	"afs",
 		net_afs,
@@ -787,7 +795,7 @@ static struct functable net_func[] = {
 		{"port",	'p', POPT_ARG_INT,    &c->opt_port},
 		{"myname",	'n', POPT_ARG_STRING, &c->opt_requester_name},
 		{"server",	'S', POPT_ARG_STRING, &c->opt_host},
-		{"encrypt",	'e', POPT_ARG_NONE,   NULL, 'e', N_("Encrypt SMB transport (UNIX extended servers only)") },
+		{"encrypt",	'e', POPT_ARG_NONE,   NULL, 'e', N_("Encrypt SMB transport") },
 		{"container",	'c', POPT_ARG_STRING, &c->opt_container},
 		{"comment",	'C', POPT_ARG_STRING, &c->opt_comment},
 		{"maxusers",	'M', POPT_ARG_INT,    &c->opt_maxusers},
@@ -831,6 +839,8 @@ static struct functable net_func[] = {
 		{"wipe", 0, POPT_ARG_NONE, &c->opt_wipe},
 		/* Options for 'net registry import' */
 		{"precheck", 0, POPT_ARG_STRING, &c->opt_precheck},
+		/* Options for 'net ads join' */
+		{"no-dns-updates", 0, POPT_ARG_NONE, &c->opt_no_dns_updates},
 		POPT_COMMON_SAMBA
 		{ 0, 0, 0, 0}
 	};
@@ -839,7 +849,7 @@ static struct functable net_func[] = {
 
 	setup_logging(argv[0], DEBUG_STDERR);
 
-	load_case_tables();
+	smb_init_locale();
 
 	setlocale(LC_ALL, "");
 #if defined(HAVE_BINDTEXTDOMAIN)
@@ -889,7 +899,23 @@ static struct functable net_func[] = {
 		}
 	}
 
-	lp_load_global(get_dyn_CONFIGFILE());
+	if (!lp_load_initial_only(get_dyn_CONFIGFILE())) {
+		d_fprintf(stderr, "Can't load %s - run testparm to debug it\n",
+			  get_dyn_CONFIGFILE());
+		exit(1);
+	}
+
+	/*
+	 * Failing to init the msg_ctx isn't a fatal error. Only root-level
+	 * things (joining/leaving domains etc.) will be denied.
+	 */
+	c->msg_ctx = messaging_init(c, samba_tevent_context_init(c));
+
+	if (!lp_load_global(get_dyn_CONFIGFILE())) {
+		d_fprintf(stderr, "Can't load %s - run testparm to debug it\n",
+			  get_dyn_CONFIGFILE());
+		exit(1);
+	}
 
 #if defined(HAVE_BIND_TEXTDOMAIN_CODESET)
 	/* Bind our gettext results to 'unix charset'
@@ -924,11 +950,11 @@ static struct functable net_func[] = {
 	}
 
 	if (!c->opt_workgroup) {
-		c->opt_workgroup = smb_xstrdup(lp_workgroup());
+		c->opt_workgroup = talloc_strdup(c, lp_workgroup());
 	}
 
 	if (!c->opt_target_workgroup) {
-		c->opt_target_workgroup = smb_xstrdup(lp_workgroup());
+		c->opt_target_workgroup = talloc_strdup(c, lp_workgroup());
 	}
 
 	if (!init_names())
@@ -953,11 +979,6 @@ static struct functable net_func[] = {
 
 	popt_burn_cmdline_password(argc, argv);
 
-	/* Failing to init the msg_ctx isn't a fatal error. Only
-	   root-level things (joining/leaving domains etc.) will be denied. */
-
-	c->msg_ctx = messaging_init(c, samba_tevent_context_init(c));
-
 	rc = net_run_function(c, argc_new-1, argv_new+1, "net", net_func);
 
 	DEBUG(2,("return code = %d\n", rc));
